@@ -66,8 +66,8 @@ ml_corr <- function(x, columns = NULL, method = c("pearson", "spearman")) {
     invoke("first") %>%
     sapply(invoke, "toArray") %>%
     matrix(nrow = num_features) %>%
-    dplyr::as_tibble() %>%
-    dplyr::rename(!!!rlang::set_names(paste0("V", seq_len(num_features)),
+    dplyr::as_tibble(.name_repair = "unique") %>%
+    dplyr::rename(!!!rlang::set_names(paste0("...", seq_len(num_features)),
                                       feature_names)
                   )
 }
