@@ -19,7 +19,8 @@ shell_connection <- function(master,
                              service,
                              remote,
                              extensions,
-                             batch) {
+                             batch,
+                             scala_version = NULL) {
   # trigger deprecated warnings
   config <- shell_connection_validate_config(config)
 
@@ -82,7 +83,8 @@ shell_connection <- function(master,
     shell_args = shell_args,
     service = service,
     remote = remote,
-    batch = batch
+    batch = batch,
+    scala_version = scala_version
   )
 }
 
@@ -138,6 +140,9 @@ abort_shell <- function(message, spark_submit_path, shell_args, output_file, err
       )
     )
   })
+
+  print("\nLocal stack trace:\n")
+  print(sys.calls())
 }
 
 # Start the Spark R Shell
@@ -153,7 +158,11 @@ start_shell <- function(master,
                         shell_args = NULL,
                         service = FALSE,
                         remote = FALSE,
-                        batch = NULL) {
+                        batch = NULL,
+                        scala_version = NULL,
+                        gateway_connect_attempts = 5,
+                        gateway_connect_retry_interval_s = 0.25,
+                        gateway_connect_retry_interval_multiplier = 2) {
 
   gatewayPort <- as.integer(spark_config_value(config, "sparklyr.gateway.port", "8880"))
   gatewayAddress <- spark_config_value(config, "sparklyr.gateway.address", "localhost")
@@ -188,7 +197,7 @@ start_shell <- function(master,
     # read app jar through config, this allows "sparkr-shell" to test sparkr backend
     app_jar <- spark_config_value(config, c("sparklyr.connect.app.jar", "sparklyr.app.jar"), NULL)
     if (is.null(app_jar)) {
-      app_jar <- spark_default_app_jar(versionSparkHome)
+      app_jar <- spark_default_app_jar(versionSparkHome, scala_version = scala_version)
       if (typeof(app_jar) != "character" || nchar(app_jar) == 0) {
         stop("sparklyr does not support Spark version: ", versionSparkHome)
       }
@@ -244,7 +253,7 @@ start_shell <- function(master,
              gsub("[-_a-zA-Z]", "", spark_version)
       )
     )
-    extensions <- spark_dependencies_from_extensions(spark_version, extensions, config)
+    extensions <- spark_dependencies_from_extensions(spark_version, scala_version, extensions, config)
 
     # combine passed jars and packages with extensions
     all_jars <- c(jars, extensions$jars)
@@ -256,20 +265,19 @@ start_shell <- function(master,
     csv_config_value <- spark_config_value(config, c("sparklyr.connect.csv.embedded", "sparklyr.csv.embedded"))
     if (!is.null(csv_config_value) &&
         length(grep(csv_config_value, spark_version)) > 0) {
+      packages <- c(packages, "com.univocity:univocity-parsers:1.5.1")
       if (spark_config_value(config, c("sparklyr.connect.csv.scala11"), FALSE)) {
-        jars <- c(
-          jars,
-          normalizePath(system.file(file.path("java", "spark-csv_2.11-1.5.0.jar"), package = "sparklyr")),
-          normalizePath(system.file(file.path("java", "commons-csv-1.5.jar"), package = "sparklyr")),
-          normalizePath(system.file(file.path("java", "univocity-parsers-1.5.1.jar"), package = "sparklyr"))
+        packages <- c(
+          packages,
+          "com.databricks:spark-csv_2.11:1.5.0",
+          "org.apache.commons:commons-csv:1.5"
         )
       }
       else {
-        jars <- c(
-          jars,
-          normalizePath(system.file(file.path("java", "spark-csv_2.10-1.5.0.jar"), package = "sparklyr")),
-          normalizePath(system.file(file.path("java", "commons-csv-1.1.jar"), package = "sparklyr")),
-          normalizePath(system.file(file.path("java", "univocity-parsers-1.5.1.jar"), package = "sparklyr"))
+        packages <- c(
+          packages,
+          "com.databricks:spark-csv_2.10:1.5.0",
+          "org.apache.commons:commons-csv:1.1"
         )
       }
     }
@@ -357,45 +365,55 @@ start_shell <- function(master,
     gatewayPort <- as.integer(spark_config_value_retries(config, "sparklyr.gateway.port", "8880", gatewayConfigRetries))
     gatewayAddress <- spark_config_value_retries(config, "sparklyr.gateway.address", "localhost", gatewayConfigRetries)
 
-    tryCatch({
-      # connect and wait for the service to start
-      gatewayInfo <- spark_connect_gateway(gatewayAddress,
-                                           gatewayPort,
-                                           sessionId,
-                                           config = config,
-                                           isStarting = TRUE)
-    }, error = function(e) {
-      abort_shell(
-        paste(
-          "Failed while connecting to sparklyr to port (",
-          gatewayPort,
-          if (spark_master_is_yarn_cluster(master, config)) {
-            paste0(
-              ") and address (",
-              gatewayAddress
-            )
-          }
-          else {
-            ""
-          },
-          ") for sessionid (",
-          sessionId,
-          "): ",
-          e$message,
-          sep = ""
-        ),
-        spark_submit_path,
-        shell_args,
-        output_file,
-        error_file
-      )
-    })
+    while (gateway_connect_attempts > 0) {
+      gateway_connect_attempts <- gateway_connect_attempts - 1
+      withCallingHandlers({
+        # connect and wait for the service to start
+        gatewayInfo <- spark_connect_gateway(gatewayAddress,
+                                             gatewayPort,
+                                             sessionId,
+                                             config = config,
+                                             isStarting = TRUE)
+        break
+      }, error = function(e) {
+        if (gateway_connect_attempts > 0) {
+          Sys.sleep(gateway_connect_retry_interval_s)
+          gateway_connect_retry_interval_s <-
+            gateway_connect_retry_interval_s * gateway_connect_retry_interval_multiplier
+        } else {
+          abort_shell(
+            paste(
+              "Failed while connecting to sparklyr to port (",
+              gatewayPort,
+              if (spark_master_is_yarn_cluster(master, config)) {
+                paste0(
+                  ") and address (",
+                  gatewayAddress
+                )
+              }
+              else {
+                ""
+              },
+              ") for sessionid (",
+              sessionId,
+              "): ",
+              e$message,
+              sep = ""
+            ),
+            spark_submit_path,
+            shell_args,
+            output_file,
+            error_file
+          )
+        }
+      })
+    }
   }
 
   # batch connections only use the shell to submit an application, not to connect.
   if (identical(batch, TRUE)) return(NULL)
 
-  tryCatch({
+  withCallingHandlers({
     interval <- spark_config_value(config, "sparklyr.backend.interval", 1)
 
     backend <- socketConnection(host = gatewayAddress,
@@ -604,7 +622,7 @@ initialize_connection.spark_shell_connection <- function(sc) {
   }
 
   # initialize and return the connection
-  tryCatch({
+  withCallingHandlers({
     backend <- invoke_static(sc, "sparklyr.Shell", "getBackend")
     sc$state$spark_context <- invoke(backend, "getSparkContext")
 
@@ -613,15 +631,18 @@ initialize_connection.spark_shell_connection <- function(sc) {
 
     init_hive_ctx_for_spark_2_plus <- function() {
       # For Spark 2.0+, we create a `SparkSession`.
-      session <- invoke_static(
+      session_builder <- invoke_static(
         sc,
         "org.apache.spark.sql.SparkSession",
         "builder"
       ) %>%
         invoke("config", conf) %>%
-        apply_config(connection_config(sc, "spark.sql."), "config", "spark.sql.") %>%
-        invoke("enableHiveSupport") %>%
-        invoke("getOrCreate")
+        apply_config(connection_config(sc, "spark.sql."), "config", "spark.sql.")
+
+      if (identical(sc$state$hive_support_enabled, TRUE))
+        invoke(session_builder, "enableHiveSupport")
+
+      session <- session_builder %>% invoke("getOrCreate")
 
       # Cache the session as the "hive context".
       sc$state$hive_context <- session
@@ -667,7 +688,15 @@ initialize_connection.spark_shell_connection <- function(sc) {
     # If Spark version is 2.0.0 or above, hive_context should be initialized by now.
     # So if that's not the case, then attempt to initialize it assuming Spark version is below 2.0.0
     sc$state$hive_context <- sc$state$hive_context %||% tryCatch(
-      invoke_new(sc, "org.apache.spark.sql.hive.HiveContext", sc$state$spark_context),
+      invoke_new(
+        sc,
+        if (identical(sc$state$hive_support_enabled, TRUE))
+          "org.apache.spark.sql.hive.HiveContext"
+        else
+          "org.apache.spark.sql.SQLContext"
+        ,
+        sc$state$spark_context
+      ),
       error = function(e) {
         warning(e$message)
         warning("Failed to create Hive context, falling back to SQL. Some operations, ",
